// updated casce.cc file for uses with Math Tablet XCAS.dll



// -*- mode:C++ ; compile-command: "g++ -I.. -I../include -g -c casce.cc" -*-
#include "first.h"
#include "windowsRqd.h"

//#include <wchar.h>

/*
*  Copyright (C) 2000 B. Parisse, Institut Fourier, 38402 St Martin d'Heres
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <stdio.h>
#include "global.h"
#ifdef HAVE_LIBREADLINE
#include <readline/readline.h>
#include <readline/history.h>
#endif // HAVE_LIBREADLINE
using namespace std;
#include <string>
#include <stdexcept>
#include <fstream>
#include <iomanip>
#include <time.h>
#include <sys/time.h>
//#include <unistd.h> // For reading arguments from file
#include <fcntl.h>
#include <cstdlib>
#include "gen.h"
#include "index.h"
#include "sym2poly.h"
#include "derive.h"
#include "intg.h"
#include "tex.h"
#include "lin.h"
#include "solve.h"
#include "modpoly.h"
#include "usual.h"
#include "sym2poly.h"
#include "moyal.h"
#include "ifactor.h"
#include "gauss.h"
#include "isom.h"
#include "plot.h"
#include "prog.h"
#include "rpn.h"
#include "pari.h"
#include "help.h"
#include "plot.h"
#include "input_lexer.h"


using namespace giac;
#ifdef HAVE_LIBREADLINE
static char *line_read = (char *)NULL;

/* Read a string, and return a pointer to it.  Returns NULL on EOF. */
char *
rl_gets (int count)
{
	/* If the buffer has already been allocated, return the memory
	to the free pool. */
	if (line_read)
	{
		free (line_read);
		line_read = (char *)NULL;
	}

	/* Get a line from the user. */
	string prompt(giac::print_INT_(count)+">> ");
	line_read = readline ((char *)prompt.c_str());

	/* If the line has any text in it, save it on the history. */
	if (line_read && *line_read)
		add_history (line_read);

	return (line_read);
}

#endif // HAVE_LIBREADLINE

void flush_stdout(){
	usleep(2000);
	fflush (stdout);
}

void flush_stderr(){
	usleep(2000);
	fflush (stderr);
}

bool has_graph_output(const giac::gen & g){
	return (g.type==giac::_SYMB) && (g._SYMBptr->sommet==giac::at_graph2tex || equalposcomp(giac::implicittex_plot_sommets,g._SYMBptr->sommet));
}

void xdvi_view_2d_plot(){
	double horiz_unit=giac::horiz_latex/(giac::gnuplot_xmax-giac::gnuplot_xmin);
	double vert_unit=giac::vert_latex/(giac::gnuplot_ymax-giac::gnuplot_ymin);
	double unit=horiz_unit;
	if (horiz_unit>vert_unit)
		unit=vert_unit;
	int i=giac::history_out.size();
	giac::graph2tex("casgraph.tex",giac::vecteur(giac::history_out.begin()+giac::erase_pos(i),giac::history_out.begin()+i),giac::gnuplot_xmin,giac::gnuplot_xmax,giac::gnuplot_ymin,giac::gnuplot_ymax,unit);
	system("latex casgraph.tex >& casgraph.___ && xdvi casgraph &") ;
}

void check_browser_help(const giac::gen & g){
	if (g.is_symb_of_sommet(giac::at_findhelp)){
		giac::gen f=g._SYMBptr->feuille;
		string s;
		if (f.type==giac::_SYMB)
			f=f._SYMBptr->sommet;
		if (f.type==giac::_FUNC)
			s=f._FUNCptr->ptr->s;
		giac::html_vtt=giac::html_help(giac::html_mtt,s);
		if (!giac::html_vtt.empty())
			system(giac::browser_command(giac::html_vtt.front()).c_str());
	}
}


// GCC 4.1 PATCH

  // GCC 4.1
// GCC 4.1 PATCH
  // variant that uses a file handle
  void readargs_from_file(HANDLE filehandle,vecteur & args,vecteur & l){
	  string to_parse;
	  char c;
	  DWORD dummy;

	  for (bool notbackslash=true;;){
		  ReadFile(filehandle,&c,1,&dummy,0);
		  //inf.get(c);
		  //if (!inf)
		  if (dummy==0)
			  break;
		  if ( notbackslash || (c!='\n') )
			  to_parse +=c;
		  else
			  to_parse = to_parse.substr(0,to_parse.size()-1);
		  notbackslash= (c!='\\');
	  }
	  gen e(to_parse,l);
	  if (e.type==_VECT)
		  args=*e._VECTptr;
	  else
		  args=makevecteur(e);
  }  
 
#define CORE_ONLY

extern "C" int WinMain(unsigned long hInstance, unsigned long hPrevInstance, TCHAR* lpCmdLine, int nCmdShow)
{
//int main(int ARGC, char *ARGV[]){    


	//cerr << giac::remove_filename(ARGV[0]) << endl;
#ifdef HAVE_LIBGSL
	gsl_set_error_handler_off();
#endif
	giac::secure_run=false;
	giac::gnuplot_ymin=-2.4;
	giac::gnuplot_ymax=2.1;


	// GCC 4.1 PATCH
	// since we are using a Windows call, this update does not support history archiving
	// The Cas session does not quit, so the history is maintained

	// casce.cc was changed to use native win32 api for reading in files
	// GCC4.1 broke the file stream functions

	// the readargs_from_stream was also patched with another variant. 

	giac::child_id=1;
	context contexte;
	context * contextptr =0;

	int session_level=-1;
	gen replace_by;

	// original stream
	//ifstream in("Cas.txt");
	//ofstream out("Out.txt");
	HANDLE infile;
	HANDLE outfile;
	DWORD ndata;
	int s;
	int i;
	string dummystring;
	char endofline[5];
	endofline[0] = '';
	endofline[1] = char(10);
	endofline[2] = char(13);
	endofline[3] = '/';
	endofline[4] = '/';


	// handle command line argument
	// argument is the startup directory


// handle command line argument
	// argument is the startup directory
	TCHAR outname[20];
	TCHAR casname[20];

	wcscpy(outname,L"Out.txt");
	wcscpy(casname,L"Cas.txt");

	bool killme=false;
	HANDLE xcasRunning;
	giac::vecteur args,l;



	xcasRunning=CreateEventW(NULL,true,false,L"XcasRunning"); // not signaled


	// cas loads and waits for an event to be signaled
	// it process the command and then reset the signal
	for (;;){

		WaitForSingleObject(xcasRunning,INFINITE);


		infile=CreateFileW(casname,GENERIC_WRITE|GENERIC_READ,FILE_SHARE_READ,NULL,OPEN_ALWAYS,0,0);

		readargs_from_file(infile,args,l);
		CloseHandle(infile);

		// no shared privilages, done to maintain compatibility with qdCAS. qdCAS uses this to figure out when
		// XCAS is done.  This is really no longer necessary since you can check the status of 
		// xcasRunning.  
		outfile=CreateFileW(outname,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,0,0);

		s=args.size();

		for ( i=0;i<s;++i){
			//out << "// " << args[i] << ' ' << char(10) << char(13) ;
		//	WriteFile(inout,args[i]._STRNGptr,10,&ndata,NULL);
			//start = clock();


// define CORE_ONLY if you don't need the echoed text in out.txt
#ifndef CORE_ONLY	
			WriteFile(outfile,endofline+3,2,&ndata,NULL);
			//dummychar=args[i].print().c_str();
			//WriteFile(outfile,dummychar,strlen(dummychar),&ndata,NULL);
			dummystring=args[i].print();
			WriteFile(outfile,dummystring.c_str(),dummystring.length(),&ndata,NULL);
			
			
			WriteFile(outfile,endofline+1,2,&ndata,NULL);
#endif


			giac::gen tmp=giac::_simplifier(giac::protecteval(args[i],giac::DEFAULT_EVAL_LEVEL,contextptr));
			//end = clock();

			if (i== (s-1)){
				// only output the results of the last value in the list
			dummystring=tmp.print();
			WriteFile(outfile,dummystring.c_str(),dummystring.length(),&ndata,NULL);


#ifndef CORE_ONLY			
			WriteFile(outfile,endofline,3,&ndata,NULL);
#else
			WriteFile(outfile,endofline+1,2,&ndata,NULL);
#endif

			//cout << dummystring.c_str();
			if (dummystring.compare(0,5,"\"QUIT") == 0) 
				killme=true; // kill the core
			}

		// original stream code
		//	out << tmp << '' <<' ' << char(10) << char(13) ;
		// out << "// Time " << double(end-start)/CLOCKS_PER_SEC << char(10) << char(13)  << char(10) << char(13) ;
		}
		CloseHandle(outfile);
		EventModify(xcasRunning,EVENT_RESET);  // not running anymore

		// the is a hack so your can kill the program nicely
		// send a "QUIT" to Cas,  quotes included
		if (killme) break; // kill the core
	}

	//delete [] casname;
	//delete [] outname;

	return 0;

}
